# R scripts paper: Black soldier fly shows a stronger preference for pig manure than for a mass-rearing diet
# Parodi et al 2020, Journal of Applied Entomology. 


#Script structure:
#Section 1 (line 20 - 105) - Repeated G-test
#Section 2 (line 109 - 152) - Log linear models
#Section 3 (line 157 - 407) - Figures

#Packages needed
library(dplyr)
library(tidyr)
library(plyr)
library(RVAideMemoire)
library(ggplot2)
library(gridExtra)
library(grid)
library(FSA)

##Section1
# Repeated G- tests
# functions obtained from: https://rcompanion.org/rcompanion/b_09.html
#Individual Chi-square
Fun.G = function (Q){                           # Functions 
  G.test(x=c(Q["Manure"], Q["Mass_rearing_diet"]),           #   to calculate
         p=c(0.5, 0.5)                  #   individual G's, 
  )$statistic                    #   df's, and p-values
}

Fun.df = function (Q){
  G.test(x=c(Q["Manure"], Q["Mass_rearing_diet"]),
         p=c(0.5, 0.5)
  )$parameter
}

Fun.p = function (Q){
  G.test(x=c(Q["Manure"], Q["Mass_rearing_diet"]),
         p=c(0.5, 0.5)
  )$p.value}


#Import data
data <- read.csv("TableS1.csv")



#remove column non-choice larvae
data[6] <- NULL

#Add new column with new names for piles
data$diet2 <- ifelse(data$pile=="M1" | data$pile=="M2", "Manure", "Mass_rearing_diet")


#Sum the values of the piles (sum two manure and sum two Mass_rearing_diet piles)
data_summary1<- ddply(data, .(replicate, treatment, day, diet2), summarize,
                         sum_piles= sum(value, na.rm=TRUE))

#re-organize columns and subset Mass_rearing_diet and Manure treatments
data_summary2 <- data_summary1 %>% spread(diet2, sum_piles)


#Repeated G-test:

all=
  mutate(data_summary2,
         Prop.Manure = Manure / (Manure + Mass_rearing_diet),                         # Calculate proportion
         #     of right arms
         G =       apply(data_summary2[c("Manure", "Mass_rearing_diet")], 1, Fun.G),
         df =      apply(data_summary2[c("Manure", "Mass_rearing_diet")], 1, Fun.df),
         p.Value = apply(data_summary2[c("Manure", "Mass_rearing_diet")], 1, Fun.p)
  )




repeated_Gtest <- all %>% 
  group_by(day, treatment) %>% 
  summarise_all(funs(sum))

repeated_Gtest$No_label <- 500 - (repeated_Gtest$Mass_rearing_diet+ repeated_Gtest$Manure)

#reorder columns
repeated_Gtest <- repeated_Gtest[c(2, 1, 5, 4, 10, 7, 8, 9)]

#reorder rows
repeated_Gtest <- repeated_Gtest[c(7, 9, 2, 4, 6, 8, 1, 3, 5),]



#change name columns
names(repeated_Gtest)[names(repeated_Gtest) == "treatment"] <- "Reared_on"
names(repeated_Gtest)[names(repeated_Gtest) == "day"] <- "Day"
names(repeated_Gtest)[names(repeated_Gtest) == "G"] <- "Total_G"

#repeated G-test
repeated_Gtest$p <- pchisq(repeated_Gtest$Total_G,
                           repeated_Gtest$df,
                           lower.tail=FALSE) 

#delete column with individual G-test p.value, and keep the repeated G-test p value
repeated_Gtest$p.Value <- NULL


#Export table 1
write.csv(repeated_Gtest, file = "Table1.csv", row.names = FALSE)



##Section 2 - log linear models

#Import and edit dataset for analysis

data2 <- read.csv("TableS1.csv")
data2[6] <- NULL
data2$age2 <- ifelse(data2$day=="Day7", "7", 
                     ifelse(data2$day=="Day9", "9",
                            ifelse(data2$day=="Day12", "12",
                                   ifelse(data2$day=="Day14", "14", "16"))))

#convert replicate and age2 to numeric class
data2$replicate <- as.numeric(as.character(data2$replicate))
data2$age2 <- as.numeric(as.character(data2$age2))

#edit dataframe
data2 <- data2[c(6, 2, 3, 4, 5)]
names(data2)[1] <- "age"
names(data2)[2] <- "replicate"
names(data2)[3] <- "start_diet"
names(data2)[5] <- "frequency"


data2 <- subset(data2, (start_diet!="Any"))

#create new column for end diet (the diet where the larvae was found)
data2$end_diet <- ifelse(data2$pile=="M1" | data2$pile=="M2", "Manure", "Mass_rearing_diet")


data3 <- subset(data2, (end_diet=="Manure"))
data4 <- subset(data2, (end_diet=="Mass_rearing_diet"))


#Log linear model for larvae found in manure
mod3 <- glm(frequency ~  age + start_diet +  replicate  + pile,
            data = data3, family = poisson(link="log"))
summary(mod3)
#cbind(mod3$data, fitted(mod3))


#Log linear model for larvae that was found in Mass_rearing_diet
mod3_1 <- glm(frequency ~  age + start_diet +  replicate  + pile,
              data = data4, family = poisson(link="log"))
summary(mod3_1)




###Section 3 - Figure

Figure2 <- read.csv("TableS1.csv")

#remove column non-choice larvae
Figure2[6] <- NULL

#Add new column with new names for piles
Figure2$diet2 <- ifelse(Figure2$pile=="M1" | Figure2$pile=="M2", "Manure", "Mass_rearing_diet")


#Sum the values of the piles (sum two manure and sum two Mass_rearing_diet piles)
Figure2_summary1<- ddply(Figure2, .(replicate, treatment, day, diet2), summarize,
                     sum_piles= sum(value, na.rm=TRUE))

#re-organize columns and subset Mass_rearing_diet and Manure treatments
Figure2_summary2 <- Figure2_summary1 %>% spread(diet2, sum_piles)


#Calculate median and quartiles
found_Mass_rearing_diet <- Summarize(Mass_rearing_diet ~ treatment + day ,
          data=Figure2_summary2,
          digits=3)

found_manure <- Summarize(Manure ~ treatment + day ,
          data=Figure2_summary2,
          digits=3)

#create dataframe with median and quartiles for those found in the Mass rearing diet
be1 <- as.data.frame(found_Mass_rearing_diet)
be1 <- be1 %>% select(1, 2, 7, 8, 9)
names(be1)[3] <- "Q1_B"
names(be1)[4] <- "median_B"
names(be1)[5] <- "Q3_B"

#dataframe with median and quartiles for those found in Manure
ma1 <- as.data.frame(found_manure)
ma1 <- ma1 %>% select(1, 2, 7, 8, 9)
names(ma1)[3] <- "Q1_M"
names(ma1)[4] <- "median_M"
names(ma1)[5] <- "Q3_M"

Figure2_update <- merge(be1, ma1)


#separate by treatments (those that were reared in the mass-rearing diet, those in manure, and for the startes that were not reared in any diet previous to the preference test)
Figure2_Mass_rearing_diet <- subset(Figure2_update, (treatment=="Mass_rearing_diet"))
Figure2_manure <- subset(Figure2_update, (treatment=="Manure"))
Figure2_any <- subset(Figure2_update, (treatment=="Any")) #for 7-days old


#this is created to add the dots to the figure
Figure2_summary111_Mass_rearing_diet  <- subset(Figure2_summary2, (treatment=="Mass_rearing_diet"))
Figure2_summary111_manure  <- subset(Figure2_summary2, (treatment=="Manure"))
Figure2_summary111_any  <- subset(Figure2_summary2, (treatment=="Any")) #for 7-days old




#Figure for that were reared on the mass-rearing diet
Figure2_Mass_rearing_diet$day <- factor(Figure2_Mass_rearing_diet$day, levels=c('Day16',
                                                    'Day14',
                                                    'Day12',
                                                    'Day9'))


b.mid <- ggplot(Figure2_Mass_rearing_diet,aes(x=1,y=day))+geom_text(aes(label=day))+
  ggtitle("")+
  ylab(NULL)+
  scale_x_continuous(expand=c(0,0),limits=c(0.94,1.065))+
  theme(axis.title=element_blank(),
        panel.grid=element_blank(),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank(),
        panel.background=element_blank(),
        axis.text.x=element_text(color=NA),
        axis.ticks.x=element_line(color=NA),
        plot.margin = unit(c(1,-1,1,-1), "mm"))



b.right <- ggplot(data = Figure2_Mass_rearing_diet, aes(x = day, y = median_B))+
  geom_bar(stat = "identity", colour="black", fill="white")+
  geom_errorbar(aes(ymin=Q1_B, ymax=Q3_B), width=.3)+
  geom_point(data=Figure2_summary111_Mass_rearing_diet,aes(day,Mass_rearing_diet), position = "jitter")+
  ggtitle("Mass_rearing_diet") +
  geom_hline(yintercept=25, linetype="dotted")+
  theme(axis.title.x = element_blank(), 
        axis.title.y = element_blank(), 
        axis.text.y = element_blank(), 
        axis.ticks.y = element_blank(),
        plot.title = element_text(hjust = 0.5),
        plot.margin = unit(c(1,-1,1,0), "mm")) +
  ylim(0, 50)+
  coord_flip()


b.left <- ggplot(data = Figure2_Mass_rearing_diet, aes(x = day, y = median_M))+
  geom_bar(stat = "identity", colour="black", fill="white")+ 
  geom_errorbar(aes(ymin=Q1_M, ymax=Q3_M), width=.3)+
  geom_point(data=Figure2_summary111_Mass_rearing_diet,aes(day,Manure), position = "jitter")+
  ggtitle("Manure") +
  geom_hline(yintercept=25, linetype="dotted")+
  theme(axis.title.x = element_blank(), 
        axis.title.y = element_blank(), 
        axis.text.y = element_blank(), 
        axis.ticks.y = element_blank(),
        plot.title = element_text(hjust = 0.5),
        plot.margin = unit(c(1,-1,1,0), "mm")) +
  scale_y_reverse() + 
  coord_flip()




gg1 <- ggplot_gtable(ggplot_build(b.right))
gg2 <- ggplot_gtable(ggplot_build(b.left))
gg.mid <- ggplot_gtable(ggplot_build(b.mid))

grid.arrange(gg2,gg.mid,gg1,ncol=3,widths=c(5,1.5,5))
a <- arrangeGrob(gg2,gg.mid,gg1,ncol=3,widths=c(5,1.5,5))
ggsave(file="Fed with Mass_rearing_diet.pdf", width = 25, height = 10, units="cm", dpi=300, a)
#Visualization were then editted in Inkscape and Adobe Illustrator.



##Figure for those reared onmanure

Figure2_manure$day <- factor(Figure2_manure$day, levels=c('Day16',
                                                            'Day14',
                                                            'Day12',
                                                            'Day9'))




m.mid <- ggplot(Figure2_manure,aes(x=1,y=day))+geom_text(aes(label=day))+
  ggtitle("")+
  ylab(NULL)+
  scale_x_continuous(expand=c(0,0),limits=c(0.94,1.065))+
  theme(axis.title=element_blank(),
        panel.grid=element_blank(),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank(),
        panel.background=element_blank(),
        axis.text.x=element_text(color=NA),
        axis.ticks.x=element_line(color=NA),
        plot.margin = unit(c(1,-1,1,-1), "mm"))



m.right <- ggplot(data = Figure2_manure, aes(x = day, y = median_B))+
  geom_bar(stat = "identity", colour="black", fill="white")+ 
  geom_errorbar(aes(ymin=Q1_B, ymax=Q3_B), width=.3)+
  geom_point(data=Figure2_summary111_manure,aes(day,Mass_rearing_diet), position = "jitter")+
  ggtitle("Mass_rearing_diet") +
  geom_hline(yintercept=25, linetype="dotted")+
  theme(axis.title.x = element_blank(), 
        axis.title.y = element_blank(), 
        axis.text.y = element_blank(), 
        axis.ticks.y = element_blank(), 
        plot.title = element_text(hjust = 0.5),
        plot.margin = unit(c(1,-1,1,0), "mm")) +
  ylim(0, 50)+
  coord_flip()


m.left <- ggplot(data = Figure2_manure, aes(x = day, y = median_M))+
  geom_bar(stat = "identity", colour="black", fill="white")+ 
  geom_errorbar(aes(ymin=Q1_M, ymax=Q3_M), width=.3)+
  geom_point(data=Figure2_summary111_manure,aes(day,Manure), position = "jitter")+
  ggtitle("Manure") +
  geom_hline(yintercept=25, linetype="dotted")+
  theme(axis.title.x = element_blank(), 
        axis.title.y = element_blank(), 
        axis.text.y = element_blank(), 
        axis.ticks.y = element_blank(), 
        plot.title = element_text(hjust = 0.5),
        plot.margin = unit(c(1,-1,1,0), "mm")) +
  scale_y_reverse() + 
  coord_flip()


mm1 <- ggplot_gtable(ggplot_build(m.right))
mm2 <- ggplot_gtable(ggplot_build(m.left))
mm.mid <- ggplot_gtable(ggplot_build(m.mid))

grid.arrange(mm2,mm.mid,mm1,ncol=3,widths=c(5,1.5,5))
b <- arrangeGrob(mm2,mm.mid,mm1,ncol=3,widths=c(5,1.5,5))
ggsave(file="Fed with manure.pdf", width = 25, height = 10, units="cm", dpi=300, b)
#Visualization were then editted in Inkscape .

##Figure for the starters that were 7-days old

a.mid <- ggplot(Figure2_any,aes(x=1,y=day))+geom_text(aes(label=day))+
  ggtitle("")+
  ylab(NULL)+
  scale_x_continuous(expand=c(0,0),limits=c(0.94,1.065))+
  theme(axis.title=element_blank(),
        panel.grid=element_blank(),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank(),
        panel.background=element_blank(),
        axis.text.x=element_text(color=NA),
        axis.ticks.x=element_line(color=NA),
        plot.margin = unit(c(1,-1,1,-1), "mm"))



a.right <- ggplot(data = Figure2_any, aes(x = day, y = median_B))+
  geom_bar(stat = "identity", colour="black", fill="white")+ 
  geom_errorbar(aes(ymin=Q1_B, ymax=Q3_B), width=.3)+
  geom_point(data=Figure2_summary111_any,aes(day,Mass_rearing_diet), position = "jitter")+
  ggtitle("Mass_rearing_diet") +
  geom_hline(yintercept=25, linetype="dotted")+
  theme(axis.title.x = element_blank(), 
        axis.title.y = element_blank(), 
        axis.text.y = element_blank(), 
        axis.ticks.y = element_blank(), 
        plot.title = element_text(hjust = 0.5),
        plot.margin = unit(c(1,-1,1,0), "mm")) +
  ylim(0, 50)+
  coord_flip()


a.left <- ggplot(data = Figure2_any, aes(x = day, y = median_M))+
  geom_bar(stat = "identity", colour="black", fill="white")+ 
  geom_errorbar(aes(ymin=Q1_M, ymax=Q3_M), width=.3)+
  geom_point(data=Figure2_summary111_any,aes(day,Manure), position = "jitter")+
  ggtitle("Manure") +
  geom_hline(yintercept=25, linetype="dotted")+
  theme(axis.title.x = element_blank(), 
        axis.title.y = element_blank(), 
        axis.text.y = element_blank(), 
        axis.ticks.y = element_blank(), 
        plot.title = element_text(hjust = 0.5),
        plot.margin = unit(c(1,-1,1,0), "mm")) +
  scale_y_reverse() + 
  coord_flip()


aa1 <- ggplot_gtable(ggplot_build(a.right))
aa2 <- ggplot_gtable(ggplot_build(a.left))
aa.mid <- ggplot_gtable(ggplot_build(a.mid))

grid.arrange(aa2,aa.mid,aa1,ncol=3,widths=c(5,1.5,5))
c <- arrangeGrob(aa2,aa.mid,aa1,ncol=3,widths=c(5,1.5,5))

ggsave(file="7-daysold.pdf", width = 25, height = 2.9, units="cm", dpi=300, c)

#Visualization were then editted in Inkscape



